/****************************************************************************
//                            Copyright (c) Microsoft Corporation.
// Purpose:
//      Demonstration script for SQL Server FileTable and Search capabilities.
//      Creates a database and associated tables. Enables full-text.
//
****************************************************************************/

-----------------------------------------------------------------------------
-- SETUP INSTRUCTIONS
-- 1. Install SQL Server with FileStream, FileTable, FullText and Semantic Search
-- 2. Specify a filegroup (SQLStorage in this script) for FileTable
-- 3. Install Semantic Language Statistics Database from the media 
--    (select SemanticLanguageDatabase.msi for your platform and language)
--    to a local folder.
-- 4. Attach this database using SSMS as any other regular dB (Datbases->Attach...)
-----------------------------------------------------------------------------


-----------------------------------------------------------------------------
-- CLEANUP
-- Clear out and create a new DB.
-----------------------------------------------------------------------------

-- Create directories used by this set of scripts.
sp_configure 'show advanced options', 1
go
reconfigure
go
execute sp_configure 'xp_cmdshell', 1
go
reconfigure
go

use master
go
if db_id(N'SQLServerArticles') is not null
begin
	alter database SQLServerArticles set single_user with rollback immediate
    drop database SQLServerArticles
end
go

execute xp_cmdshell 'rd /s /q C:\Files\SQLServerArticles', no_output
execute xp_cmdshell 'md C:\Files\SQLServerArticles', no_output
go

-----------------------------------------------------------------------------
-- REGISTER SEMANTICS LANGUAGES DATABSE (ONE TIME ONLY)
--
-----------------------------------------------------------------------------
EXEC sp_fulltext_semantic_register_language_statistics_db 'semanticsdb';
GO

/* Verify the registration is succeeded */
SELECT * FROM sys.fulltext_semantic_language_statistics_database;
GO

/* Check available languages for statistical semantic extraction */
SELECT * FROM sys.fulltext_semantic_languages;
GO

-----------------------------------------------------------------------------
-- STEP 1: CREATE/ALTER DATABASE
--
-- Create a database enabled for non-transactional access in existing filegroup
-- ("SQLStorage") with user friendly root folder name ("SQLServer Articles")
-----------------------------------------------------------------------------

create database SQLServerArticles
on primary (name = SQLServerArticles_File, filename = N'C:\Files\SQLServerArticles\SQLServerArticles_File.mdf' ),
filegroup SQLStorage contains filestream ( name = SQLServerArticles_FS_File, filename = N'C:\Files\SQLServerArticles\SQLServerArticles_FS' )
with filestream ( 
    non_transacted_access = full,
    directory_name = N'SQLServer Articles' )
go

-----------------------------------------------------------------------------
-- STEP 2: CREATE A FILETABLE(s)
--
-- Create FileTable (Documents) enabled for non-transactional access
-- with user friendly folder name (Document Library)
--
-----------------------------------------------------------------------------

create table SQLServerArticles..Documents as FileTable with (filetable_directory = N'Document Library')
go

select * from SQLServerArticles..Documents

-----------------------------------------------------------------------------
-- STEP 3: Create FullText Index on Documents Table
--			a. Create a fulltext catalog
--			b. Create a unique index on Documents table
--			c. Create a Property List with Title, Author and Tags properties
--			d. Create fulltext Index with Semantics and Properties search
-----------------------------------------------------------------------------

USE SQLServerArticles
GO

/*
-- Don't forget to register ifilters
-- check existing ifilters
EXEC sp_help_fulltext_system_components 'filter';  
GO

-- after installing new ifilters, refresh
exec sp_fulltext_service 'update_languages'; 
GO

-- Reload filters and restart fulltext host process if needed
EXEC sp_fulltext_service 'load_os_resources', 1;
EXEC sp_fulltext_service 'restart_all_fdhosts';
GO

-- which document types are supported by FTS?
select * from sys.fulltext_document_types

--alter fulltext index on SQLServerArticles..Documents start update population
--go
*/

CREATE FULLTEXT CATALOG ft AS DEFAULT
GO

CREATE UNIQUE INDEX DocumentsFt ON SQLServerArticles..Documents(stream_id)
GO

CREATE SEARCH PROPERTY LIST DocumentProperties;
GO
ALTER SEARCH PROPERTY LIST DocumentProperties
   ADD 'Title'
   WITH ( PROPERTY_SET_GUID = 'F29F85E0-4FF9-1068-AB91-08002B27B3D9', PROPERTY_INT_ID = 2, 
      PROPERTY_DESCRIPTION = 'System.Title - Title of the item.' );
GO
ALTER SEARCH PROPERTY LIST DocumentProperties
    ADD 'Author'
   WITH ( PROPERTY_SET_GUID = 'F29F85E0-4FF9-1068-AB91-08002B27B3D9', PROPERTY_INT_ID = 4, 
      PROPERTY_DESCRIPTION = 'System.Author - Author or authors of the item.' );
GO
ALTER SEARCH PROPERTY LIST DocumentProperties 
    ADD 'Tags'
   WITH ( PROPERTY_SET_GUID = 'F29F85E0-4FF9-1068-AB91-08002B27B3D9', PROPERTY_INT_ID = 5, 
      PROPERTY_DESCRIPTION = 'System.Keywords - Keywords (Tags) of the item.' );
GO

-- Add a full text index, indexing the file_stream column and using the file_type computed column as the type
-- Specify statistical semantics for key phrase and similarity extractions along with fulltext keywords
-- Specify DocumentProperties Property List for properties extraction

CREATE FULLTEXT INDEX ON SQLServerArticles..Documents
	(file_stream TYPE COLUMN file_type LANGUAGE 1033 statistical_semantics)
	KEY INDEX DocumentsFt ON ft
	WITH SEARCH PROPERTY LIST = DocumentProperties
GO

/* Verify indexed columns for new Semantic Index metadata */
SELECT OBJECT_NAME(object_id) TableName, COL_NAME(object_id, column_id) IndexedColumn, *
FROM sys.fulltext_index_columns
GO

/* Verify new Semantic Index metadata on indexed tables and columns */
SELECT OBJECTPROPERTYEX(OBJECT_ID('SQLServerArticles..Documents'), 'TableFullTextSemanticExtraction')
SELECT COLUMNPROPERTY(OBJECT_ID('SQLServerArticles..Documents'), 'file_stream', 'StatisticalSemantics')
GO

-----------------------------------------------------------------------------
--
-- TRANSFER FILES (source: C:\DemoScripts\SQLServerArticlesSet) USING
-- SHELL EXPLORER INTO NEWLY CREATED DOCUMENT LIBRARY
--
-----------------------------------------------------------------------------

-- Show Files in FileTable using T-SQL
--
SELECT FileTableRootPath() AS 'FileTable Root Path'
GO

SELECT stream_id, name, file_type, file_stream.GetFileNamespacePath() FilePath,
	creation_Time, last_write_time, last_access_time, is_directory, is_offline,
	is_hidden, is_readonly, is_archive, is_system, is_temporary
	FROM SQLServerArticles..Documents
	ORDER BY name ASC
GO

-----------------------------------------------------------------------------
--
-- CHECK INDEX POPULATION STATUS
--
-----------------------------------------------------------------------------
/* Check Population Progress */
SELECT fulltextcatalogproperty('ft', 'populatestatus')
GO

SELECT DB_NAME(database_id) AS 'DB Name', OBJECT_NAME(table_id) AS 'Table Name',
	population_type, status, status_description, worker_count, start_time, *
	FROM sys.dm_fts_index_population

SELECT DB_NAME(database_id) AS 'DB Name', OBJECT_NAME(table_id) AS 'Table Name',
	status, status_description, worker_count, start_time, *
	FROM sys.dm_fts_semantic_similarity_population

/* check Population size */
SELECT OBJECT_NAME(object_id), fulltext_index_page_count,
	keyphrase_index_page_count, similarity_index_page_count
	FROM sys.dm_db_fts_index_physical_stats
GO

-----------------------------------------------------------------------------
--
-- QUERY THE RESULTS
--
-----------------------------------------------------------------------------

-----------------------------------------------------------------------------
-- 1. USAGE OF PROPERTY Scoped search within CONTAINS
-----------------------------------------------------------------------------

/* Search for properties only */
SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Documents
	WHERE CONTAINS(PROPERTY(file_stream, 'Title'), 'data OR SQL');
GO

-----------------------------------------------------------------------------
-- 2. USAGE OF NEAR Operator within CONTAINS
-----------------------------------------------------------------------------

/* Get documents that contain keywords "data" and "SQL" nearby */
SELECT name DocumentName, file_stream.GetFileNamespacePath() Path FROM Documents
	WHERE CONTAINS(file_stream, 'NEAR(("data", "SQL"), 5, FALSE)');
GO

-----------------------------------------------------------------------------
-- 3. USAGE OF SEMANTICKEYPHRASETABLE
-----------------------------------------------------------------------------

/* Get top key phrases in the entire corpus */
SELECT name, document_key, keyphrase, score
	FROM semantickeyphrasetable(Documents, *)
	INNER JOIN Documents ON stream_id = document_key
	ORDER BY name, score DESC
Go

/* Get key phrases for Data Compression Paper */
USE SQLServerArticles
DECLARE @Title as NVARCHAR(1000)
DECLARE @DocID as UNIQUEIDENTIFIER

SET	@Title = 'DataCompression.pdf'
--SET	@Title = 'BestPractDWSQL2008.docx'

SELECT @DocID = stream_id FROM Documents WHERE name = @Title

SELECT name, document_key, keyphrase, score
	FROM semantickeyphrasetable(Documents, *, @DocID)
	INNER JOIN Documents ON stream_id = document_key
	ORDER BY name, score DESC
Go

-----------------------------------------------------------------------------
-- 4. USAGE OF SEMANTICSIMILARITYTABLE
-----------------------------------------------------------------------------

/* Get similar documents for Data Compression Paper */
USE SQLServerArticles
DECLARE @Title as NVARCHAR(1000)
DECLARE @DocID as UNIQUEIDENTIFIER

SET	@Title = 'DataCompression.pdf'
--SET	@Title = 'BestPractDWSQL2008.docx'

SELECT @DocID = stream_id FROM Documents WHERE name = @Title

SELECT @Title AS SourceTitle, name AS MatchedTitle, stream_id, score
	FROM semanticsimilaritytable(Documents, *, @DocID)
	INNER JOIN Documents ON stream_id = matched_document_key
	ORDER BY score DESC
GO

-----------------------------------------------------------------------------
-- 5. USAGE OF SEMANTICSIMILARITYDETAILSTABLE
-----------------------------------------------------------------------------

/* Get Similarity Details for one of the matched Document */
USE SQLServerArticles
DECLARE @SourceTitle as NVARCHAR(1000)
DECLARE @MatchedTitle as NVARCHAR(1000)
DECLARE @SourceDocID as UNIQUEIDENTIFIER
DECLARE @MatchedDocID as UNIQUEIDENTIFIER

SET	@SourceTitle = 'DataCompression.pdf'
SET @MatchedTitle = 'ScaleUpDWinSQL2008.docx'
--SET @SourceTitle = 'BestPractDWSQL2008.docx'

SELECT @SourceDocID = stream_id FROM Documents WHERE name = @SourceTitle
SELECT @MatchedDocID = stream_id FROM Documents WHERE name = @MatchedTitle

SELECT @SourceTitle AS SourceTitle, @MatchedTitle AS MatchedTitle, keyphrase, score
	FROM semanticsimilaritydetailstable(Documents, file_stream, @SourceDocID, file_stream, @MatchedDocID)
	ORDER BY score DESC
GO

select * from ftswb.sys.internal_tables
select * from sys.internal_tables

select * from sys.fulltext_index_docidmap_245575913

select * from sys.indexes

select * from sys.dm_fts_index_keywords_by_document( DB_ID('SQLServerArticles'), OBJECT_ID('Documents'))

select * from sys.columns
where object_id = 245575913








